@@ -114,15 +114,15 @@ afsi_log(char *pattern, ...)
     if ( afsi_log_useTimestamp ) {
         GetTimeFormat(LOCALE_SYSTEM_DEFAULT, 0, NULL, NULL, t, sizeof(t));
         GetDateFormat(LOCALE_SYSTEM_DEFAULT, 0, NULL, NULL, d, sizeof(d));
-        StringCbPrintfA(u, sizeof(u), "%s %s: %s\r\n", d, t, s);
+        StringCbPrintfA(u, sizeof(u), "%s %s: %s\n", d, t, s);
         if (afsi_file != INVALID_HANDLE_VALUE)
-            WriteFile(afsi_file, u, strlen(u), &zilch, NULL);
+            WriteFile(afsi_file, u, (DWORD)strlen(u), &zilch, NULL);
 #ifdef NOTSERVICE
         printf("%s", u);
 #endif 
     } else {
         if (afsi_file != INVALID_HANDLE_VALUE)
-            WriteFile(afsi_file, s, strlen(s), &zilch, NULL);
+            WriteFile(afsi_file, s, (DWORD)strlen(s), &zilch, NULL);
     }
 }
 
@@ -179,15 +179,15 @@ afsi_start()
     GetTimeFormat(LOCALE_SYSTEM_DEFAULT, 0, NULL, NULL, u, sizeof(u));
     StringCbCatA(t, sizeof(t), ": Create log file\n");
     StringCbCatA(u, sizeof(u), ": Created log file\n");
-    WriteFile(afsi_file, t, strlen(t), &zilch, NULL);
-    WriteFile(afsi_file, u, strlen(u), &zilch, NULL);
+    WriteFile(afsi_file, t, (DWORD)strlen(t), &zilch, NULL);
+    WriteFile(afsi_file, u, (DWORD)strlen(u), &zilch, NULL);
     p = "PATH=";
     code = GetEnvironmentVariable("PATH", NULL, 0);
     path = malloc(code);
     code = GetEnvironmentVariable("PATH", path, code);
-    WriteFile(afsi_file, p, strlen(p), &zilch, NULL);
-    WriteFile(afsi_file, path, strlen(path), &zilch, NULL);
-    WriteFile(afsi_file, "\n", 1, &zilch, NULL);
+    WriteFile(afsi_file, p, (DWORD)strlen(p), &zilch, NULL);
+    WriteFile(afsi_file, path, (DWORD)strlen(path), &zilch, NULL);
+    WriteFile(afsi_file, "\n", (DWORD)1, &zilch, NULL);
     free(path);
 
     /* Initialize C RTL Code Page conversion functions */
@@ -268,7 +268,7 @@ configureBackConnectionHostNames(void)
     HKEY hkMSV10;
     HKEY hkClient;
     DWORD dwType;
-    DWORD dwSize = 0, dwAllocSize = 0;
+    DWORD dwSize;
     DWORD dwValue;
     PBYTE pHostNames = NULL, pName = NULL;
     BOOL  bNameFound = FALSE;   
@@ -279,17 +279,13 @@ configureBackConnectionHostNames(void)
                        KEY_READ|KEY_WRITE,
                        &hkMSV10) == ERROR_SUCCESS )
     {
-	if (RegQueryValueEx( hkMSV10, "BackConnectionHostNames", 0, 
-			     &dwType, NULL, &dwAllocSize) == ERROR_SUCCESS) {
-	    dwAllocSize += 1 /* in case the source string is not nul terminated */
-		+ strlen(cm_NetbiosName) + 2;
-	    pHostNames = malloc(dwAllocSize);
-	    dwSize = dwAllocSize;
-	    if (RegQueryValueEx( hkMSV10, "BackConnectionHostNames", 0, &dwType, 
+        if (RegQueryValueEx( hkMSV10, "BackConnectionHostNames", 0, 
+			     &dwType, NULL, &dwSize) == ERROR_SUCCESS) {
+	    dwSize += strlen(cm_NetbiosName) + 1;
+	    pHostNames = malloc(dwSize);
+            if (RegQueryValueEx( hkMSV10, "BackConnectionHostNames", 0, &dwType, 
 				 pHostNames, &dwSize) == ERROR_SUCCESS) {
-		for (pName = pHostNames; 
-		     (pName - pHostNames < dwSize) && *pName; 
-		     pName += strlen(pName) + 1)
+		for (pName = pHostNames; *pName ; pName += strlen(pName) + 1)
 		{
 		    if ( !stricmp(pName, cm_NetbiosName) ) {
 			bNameFound = TRUE;
@@ -297,12 +293,13 @@ configureBackConnectionHostNames(void)
 		    }   
 		}
 	    }
-	}
+        }
              
         if ( !bNameFound ) {
-            int size = strlen(cm_NetbiosName) + 2;
+            size_t size = strlen(cm_NetbiosName) + 2;
             if ( !pHostNames ) {
                 pHostNames = malloc(size);
+		dwSize = size;
 		pName = pHostNames;
             }
             StringCbCopyA(pName, size, cm_NetbiosName);
@@ -310,7 +307,6 @@ configureBackConnectionHostNames(void)
             *pName = '\0';  /* add a second nul terminator */
 
             dwType = REG_MULTI_SZ;
-            dwSize = pName - pHostNames + 1;
             RegSetValueEx( hkMSV10, "BackConnectionHostNames", 0, dwType, pHostNames, dwSize);
 
             if ( RegOpenKeyEx( HKEY_LOCAL_MACHINE, 
@@ -541,10 +537,11 @@ static void afsd_InitServerPreferences(void)
 int afsd_InitCM(char **reasonP)
 {
     osi_uid_t debugID;
-    DWORD cacheBlocks;
+    afs_uint64 cacheBlocks;
     DWORD cacheSize;
     long logChunkSize;
     DWORD stats;
+    DWORD dwValue;
     DWORD rx_enable_peer_stats = 0;
     DWORD rx_enable_process_stats = 0;
     long traceBufSize;
@@ -667,7 +664,7 @@ int afsd_InitCM(char **reasonP)
 
     /* setup and enable debug log */
     afsd_logp = osi_LogCreate("afsd", traceBufSize);
-    afsi_log("osi_LogCreate log addr %x", (int)afsd_logp);
+    afsi_log("osi_LogCreate log addr %x", PtrToUlong(afsd_logp));
     if ((TraceOption & 0x8)
 #ifdef DEBUG
 	 || 1
@@ -771,7 +768,7 @@ int afsd_InitCM(char **reasonP)
                             cm_mountRoot, &cm_mountRootLen);
     if (code == ERROR_SUCCESS) {
         afsi_log("Mount root %s", cm_mountRoot);
-        cm_mountRootLen = strlen(cm_mountRoot);
+        cm_mountRootLen = (DWORD)strlen(cm_mountRoot);
     } else {
         StringCbCopyA(cm_mountRoot, sizeof(cm_mountRoot), "/afs");
         cm_mountRootLen = 4;
@@ -893,6 +890,11 @@ int afsd_InitCM(char **reasonP)
         afsi_log("Default SecurityLevel is clear");
     }
 
+    if (cryptall)
+	LogEvent(EVENTLOG_INFORMATION_TYPE, MSG_CRYPT_ON);
+    else
+	LogEvent(EVENTLOG_INFORMATION_TYPE, MSG_CRYPT_OFF);
+
 #ifdef AFS_AFSDB_ENV
     dummyLen = sizeof(cm_dnsEnabled);
     code = RegQueryValueEx(parmKey, "UseDNS", NULL, NULL,
@@ -922,25 +924,6 @@ int afsd_InitCM(char **reasonP)
     }
 #endif /* AFS_FREELANCE_CLIENT */
 
-#ifdef COMMENT
-    /* The netbios name is looked up in lana_GetUNCServerNameEx */
-    dummyLen = sizeof(buf);
-    code = RegQueryValueEx(parmKey, "NetbiosName", NULL, NULL,
-                           (BYTE *) &buf, &dummyLen);
-    if (code == ERROR_SUCCESS) {
-        DWORD len = ExpandEnvironmentStrings(buf, cm_NetbiosName, MAX_NB_NAME_LENGTH);
-        if ( len > 0 && len <= MAX_NB_NAME_LENGTH ) {
-            afsi_log("Explicit NetBios name is used %s", cm_NetbiosName);
-        } else {
-            afsi_log("Unable to Expand Explicit NetBios name: %s", buf);
-            cm_NetbiosName[0] = 0;  /* turn it off */
-        }
-    }
-    else {
-        cm_NetbiosName[0] = 0;   /* default off */
-    }
-#endif
-
     dummyLen = sizeof(smb_hideDotFiles);
     code = RegQueryValueEx(parmKey, "HideDotFiles", NULL, NULL,
                            (BYTE *) &smb_hideDotFiles, &dummyLen);
@@ -1022,6 +1005,48 @@ int afsd_InitCM(char **reasonP)
                            (BYTE *) &HardDeadtimeout, &dummyLen);
     afsi_log("HardDeadTimeout is %d", HardDeadtimeout);
 
+    dummyLen = sizeof(DWORD);
+    code = RegQueryValueEx(parmKey, "daemonCheckDownInterval", NULL, NULL,
+			    (BYTE *) &dwValue, &dummyLen);
+    if (code == ERROR_SUCCESS)
+	cm_daemonCheckDownInterval = dwValue;
+    afsi_log("daemonCheckDownInterval is %d", cm_daemonCheckDownInterval);
+
+    dummyLen = sizeof(DWORD);
+    code = RegQueryValueEx(parmKey, "daemonCheckUpInterval", NULL, NULL,
+			    (BYTE *) &dwValue, &dummyLen);
+    if (code == ERROR_SUCCESS)
+	cm_daemonCheckUpInterval = dwValue;
+    afsi_log("daemonCheckUpInterval is %d", cm_daemonCheckUpInterval);
+
+    dummyLen = sizeof(DWORD);
+    code = RegQueryValueEx(parmKey, "daemonCheckVolInterval", NULL, NULL,
+			    (BYTE *) &dwValue, &dummyLen);
+    if (code == ERROR_SUCCESS)
+	cm_daemonCheckVolInterval = dwValue;
+    afsi_log("daemonCheckVolInterval is %d", cm_daemonCheckVolInterval);
+
+    dummyLen = sizeof(DWORD);
+    code = RegQueryValueEx(parmKey, "daemonCheckCBInterval", NULL, NULL,
+			    (BYTE *) &dwValue, &dummyLen);
+    if (code == ERROR_SUCCESS)
+	cm_daemonCheckCBInterval = dwValue;
+    afsi_log("daemonCheckCBInterval is %d", cm_daemonCheckCBInterval);
+
+    dummyLen = sizeof(DWORD);
+    code = RegQueryValueEx(parmKey, "daemonCheckLockInterval", NULL, NULL,
+			    (BYTE *) &dwValue, &dummyLen);
+    if (code == ERROR_SUCCESS)
+	cm_daemonCheckLockInterval = dwValue;
+    afsi_log("daemonCheckLockInterval is %d", cm_daemonCheckLockInterval);
+
+    dummyLen = sizeof(DWORD);
+    code = RegQueryValueEx(parmKey, "daemonCheckTokenInterval", NULL, NULL,
+			    (BYTE *) &dwValue, &dummyLen);
+    if (code == ERROR_SUCCESS)
+	cm_daemonTokenCheckInterval = dwValue;
+    afsi_log("daemonCheckTokenInterval is %d", cm_daemonTokenCheckInterval);
+
     RegCloseKey (parmKey);
 
     /* Call lanahelper to get Netbios name, lan adapter number and gateway flag */
@@ -1043,7 +1068,7 @@ int afsd_InitCM(char **reasonP)
         osi_panic(buf, __FILE__, __LINE__);
     }
 
-    cacheBlocks = (cacheSize * 1024) / CM_CONFIGDEFAULT_BLOCKSIZE;
+    cacheBlocks = ((afs_uint64)cacheSize * 1024) / CM_CONFIGDEFAULT_BLOCKSIZE;
         
     /* get network related info */
     cm_noIPAddr = CM_MAXINTERFACE_ADDR;
@@ -1062,7 +1087,7 @@ int afsd_InitCM(char **reasonP)
      */
     cm_initParams.nChunkFiles = 0;
     cm_initParams.nStatCaches = stats;
-    cm_initParams.nDataCaches = cacheBlocks;
+    cm_initParams.nDataCaches = (afs_uint32)(cacheBlocks > 0xFFFFFFFF ? 0xFFFFFFFF : cacheBlocks);
     cm_initParams.nVolumeCaches = stats/2;
     cm_initParams.firstChunkSize = cm_chunkSize;
     cm_initParams.otherChunkSize = cm_chunkSize;
@@ -1126,7 +1151,7 @@ int afsd_InitCM(char **reasonP)
     nullServerSecurityClassp = rxnull_NewServerSecurityObject();
     serverp = rx_NewService(0, 1, "AFS", &nullServerSecurityClassp, 1,
                              RXAFSCB_ExecuteRequest);
-    afsi_log("rx_NewService addr %x", (int)serverp);
+    afsi_log("rx_NewService addr %x", PtrToUlong(serverp));
     if (serverp == NULL) {
         *reasonP = "unknown error";
         return -1;
@@ -1135,7 +1160,7 @@ int afsd_InitCM(char **reasonP)
     nullServerSecurityClassp = rxnull_NewServerSecurityObject();
     serverp = rx_NewService(0, RX_STATS_SERVICE_ID, "rpcstats",
                              &nullServerSecurityClassp, 1, RXSTATS_ExecuteRequest);
-    afsi_log("rx_NewService addr %x", (int)serverp);
+    afsi_log("rx_NewService addr %x", PtrToUlong(serverp));
     if (serverp == NULL) {
         *reasonP = "unknown error";
         return -1;
@@ -1165,7 +1190,7 @@ int afsd_InitCM(char **reasonP)
     if (code == 0 && !cm_freelanceEnabled) 
     {
         cm_data.rootCellp = cm_GetCell(rootCellName, CM_FLAG_CREATE);
-        afsi_log("cm_GetCell addr %x", (int)cm_data.rootCellp);
+        afsi_log("cm_GetCell addr %x", PtrToUlong(cm_data.rootCellp));
         if (cm_data.rootCellp == NULL) 
         {
             *reasonP = "can't find root cell in afsdcell.ini";
@@ -1187,13 +1212,6 @@ int afsd_InitCM(char **reasonP)
     return 0;
 }
 
-int afsd_ShutdownCM(void)
-{
-    cm_ReleaseSCache(cm_data.rootSCachep);
-
-    return 0;
-}
-
 int afsd_InitDaemons(char **reasonP)
 {
     long code;
@@ -1204,15 +1222,11 @@ int afsd_InitDaemons(char **reasonP)
     /* this should really be in an init daemon from here on down */
 
     if (!cm_freelanceEnabled) {
-	int attempts = 10;
-
         osi_Log0(afsd_logp, "Loading Root Volume from cell");
-	do {
-	    code = cm_GetVolumeByName(cm_data.rootCellp, cm_rootVolumeName, cm_rootUserp,
-				       &req, CM_FLAG_CREATE, &cm_data.rootVolumep);
-	    afsi_log("cm_GetVolumeByName code %x root vol %x", code,
-		      (code ? (cm_volume_t *)-1 : cm_data.rootVolumep));
-	} while (code && --attempts);
+        code = cm_GetVolumeByName(cm_data.rootCellp, cm_rootVolumeName, cm_rootUserp,
+                                  &req, CM_FLAG_CREATE, &cm_data.rootVolumep);
+        afsi_log("cm_GetVolumeByName code %x root vol %x", code,
+                 (code ? (cm_volume_t *)-1 : cm_data.rootVolumep));
         if (code != 0) {
             *reasonP = "can't find root volume in root cell";
             return -1;
@@ -1286,9 +1300,14 @@ int afsd_InitSMB(char **reasonP, void *aMBfunc)
 
 void afsd_printStack(HANDLE hThread, CONTEXT *c)
 {
+#if defined(_X86_)
     HANDLE hProcess = GetCurrentProcess();
     int frameNum;
+#if defined(_AMD64_)
+    DWORD64 offset;
+#elif defined(_X86_)
     DWORD offset;
+#endif
     DWORD symOptions;
     char functionName[MAXNAMELEN];
   
@@ -1327,6 +1346,11 @@ void afsd_printStack(HANDLE hThread, CONTEXT *c)
 #if defined (_ALPHA_) || defined (_MIPS_) || defined (_PPC_)
 #error The STACKFRAME initialization in afsd_printStack() for this platform
 #error must be properly configured
+#elif defined(_AMD64_)
+    s.AddrPC.Offset = 0;
+    s.AddrPC.Mode = AddrModeFlat;
+    s.AddrFrame.Offset = 0;
+    s.AddrFrame.Mode = AddrModeFlat;
 #else
     s.AddrPC.Offset = c->Eip;
     s.AddrPC.Mode = AddrModeFlat;
@@ -1425,6 +1449,7 @@ void afsd_printStack(HANDLE hThread, CONTEXT *c)
   
     SymCleanup(hProcess);
     GlobalFree(pSym);
+#endif /* _X86_ */
 }
 
 #ifdef _DEBUG
@@ -1494,7 +1519,7 @@ GenerateMiniDump(PEXCEPTION_POINTERS ep)
             DWORD dummyLen;
             DWORD dwValue;
             DWORD code;
-            DWORD dwMiniDumpType = MiniDumpWithDataSegs;
+            DWORD dwMiniDumpType = MiniDumpNormal;
 
             code = RegOpenKeyEx(HKEY_LOCAL_MACHINE, AFSREG_CLT_OPENAFS_SUBKEY,
                                  0, KEY_QUERY_VALUE, &parmKey);
@@ -1503,7 +1528,7 @@ GenerateMiniDump(PEXCEPTION_POINTERS ep)
                 code = RegQueryValueEx(parmKey, "MiniDumpType", NULL, NULL,
                                         (BYTE *) &dwValue, &dummyLen);
                 if (code == ERROR_SUCCESS)
-                    dwMiniDumpType = dwValue;
+                    dwMiniDumpType = dwValue ? 1 : 0;
                 RegCloseKey (parmKey);
             }
 
@@ -1568,8 +1593,9 @@ LONG __stdcall afsd_ExceptionFilter(EXCEPTION_POINTERS *ep)
             _CrtSetBreakAlloc(*afsd_crtDbgBreakCurrent);
         }
 #endif         
-    
+#if defined(_X86)    
         ep->ContextRecord->Eip++;
+#endif
         return EXCEPTION_CONTINUE_EXECUTION;
     }
     else
