@@ -21,6 +21,8 @@
 #include <afsconfig.h>
 #include "afs/param.h"
 
+RCSID
+    ("$Header: /cvs/openafs/src/afs/LINUX/osi_vnodeops.c,v 1.124 2006/01/11 21:37:06 shadow Exp $");
 
 #include "afs/sysincludes.h"
 #include "afsincludes.h"
@@ -43,38 +45,13 @@
 #define pageoff(pp) pp->offset
 #endif
 
-#ifndef MAX_ERRNO
-#define MAX_ERRNO 1000L
-#endif
-
 #if defined(AFS_LINUX26_ENV)
 #define UnlockPage(pp) unlock_page(pp)
-extern struct backing_dev_info *afs_backing_dev_info;
-#if !defined(WRITEPAGE_ACTIVATE)
-#define WRITEPAGE_ACTIVATE AOP_WRITEPAGE_ACTIVATE
-#endif
 #endif
 
 extern struct vcache *afs_globalVp;
-extern int afs_notify_change(struct dentry *dp, struct iattr *iattrp);
 
 
-
-/* This function converts a positive error code from AFS into a negative
- * code suitable for passing into the Linux VFS layer. It checks that the
- * error code is within the permissable bounds for the ERR_PTR mechanism.
- *
- * _All_ error codes which come from the AFS layer should be passed through
- * this function before being returned to the kernel.
- */
-
-static inline int afs_convert_code(int code) {
-    if ((code >= 0) && (code <= MAX_ERRNO))
-	return -code;
-    else
-	return -EIO;
-}
-
 static ssize_t
 afs_linux_read(struct file *fp, char *buf, size_t count, loff_t * offp)
 {
@@ -94,15 +71,11 @@ afs_linux_read(struct file *fp, char *buf, size_t count, loff_t * offp)
 	code = afs_VerifyVCache(vcp, &treq);
 
     if (code)
-	code = afs_convert_code(code);
+	code = -code;
     else {
 	    osi_FlushPages(vcp, credp);	/* ensure stale pages are gone */
 	    AFS_GUNLOCK();
-#ifdef DO_SYNC_READ
-	    code = do_sync_read(fp, buf, count, offp);
-#else
 	    code = generic_file_read(fp, buf, count, offp);
-#endif
 	    AFS_GLOCK();
     }
 
@@ -124,6 +97,7 @@ static ssize_t
 afs_linux_write(struct file *fp, const char *buf, size_t count, loff_t * offp)
 {
     ssize_t code = 0;
+    int code2 = 0;
     struct vcache *vcp = VTOAFS(fp->f_dentry->d_inode);
     struct vrequest treq;
     cred_t *credp = crref();
@@ -144,14 +118,10 @@ afs_linux_write(struct file *fp, const char *buf, size_t count, loff_t * offp)
     afs_FakeOpen(vcp);
     ReleaseWriteLock(&vcp->lock);
     if (code)
-	code = afs_convert_code(code);
+	code = -code;
     else {
 	    AFS_GUNLOCK();
-#ifdef DO_SYNC_READ
-	    code = do_sync_write(fp, buf, count, offp);
-#else
 	    code = generic_file_write(fp, buf, count, offp);
-#endif
 	    AFS_GLOCK();
     }
 
@@ -199,25 +169,19 @@ afs_linux_readdir(struct file *fp, void *dirbuf, filldir_t filldir)
 
     code = afs_InitReq(&treq, credp);
     crfree(credp);
-    if (code) {
-	code = afs_convert_code(code);
+    if (code)
 	goto out1;
-    }
 
     afs_InitFakeStat(&fakestat);
     code = afs_EvalFakeStat(&avc, &fakestat, &treq);
-    if (code) {
-	code = afs_convert_code(code);
+    if (code)
 	goto out;
-    }
 
     /* update the cache entry */
   tagain:
     code = afs_VerifyVCache(avc, &treq);
-    if (code) {
-	code = afs_convert_code(code);
+    if (code)
 	goto out;
-    }
 
     /* get a reference to the entire directory */
     tdc = afs_GetDCache(avc, (afs_size_t) 0, &treq, &origOffset, &tlen, 1);
@@ -311,9 +275,7 @@ afs_linux_readdir(struct file *fp, void *dirbuf, filldir_t filldir)
 		/* clean up from afs_FindVCache */
 		afs_PutVCache(tvc);
 	    }
-	    AFS_GUNLOCK();
 	    code = (*filldir) (dirbuf, de->name, len, offset, ino, type);
-	    AFS_GLOCK();
 	}
 #else
 	code = (*filldir) (dirbuf, de->name, len, offset, ino);
@@ -400,7 +362,7 @@ out:
     return code;
 
 out_err:
-    code = afs_convert_code(code);
+    code = -code;
     goto out;
 }
 
@@ -422,7 +384,7 @@ afs_linux_open(struct inode *ip, struct file *fp)
 #endif
 
     crfree(credp);
-    return afs_convert_code(code);
+    return -code;
 }
 
 static int
@@ -443,14 +405,12 @@ afs_linux_release(struct inode *ip, struct file *fp)
 #endif
 
     crfree(credp);
-    return afs_convert_code(code);
+    return -code;
 }
 
 static int
-#if defined(AFS_LINUX24_ENV) && defined(FOP_FSYNC_TAKES_DENTRY)
+#if defined(AFS_LINUX24_ENV)
 afs_linux_fsync(struct file *fp, struct dentry *dp, int datasync)
-#elif defined(AFS_LINUX24_ENV)
-afs_linux_fsync(struct file *fp, int datasync)
 #else
 afs_linux_fsync(struct file *fp, struct dentry *dp)
 #endif
@@ -469,7 +429,7 @@ afs_linux_fsync(struct file *fp, struct dentry *dp)
     unlock_kernel();
 #endif
     crfree(credp);
-    return afs_convert_code(code);
+    return -code;
 
 }
 
@@ -481,111 +441,13 @@ afs_linux_lock(struct file *fp, int cmd, struct file_lock *flp)
     struct vcache *vcp = VTOAFS(FILE_INODE(fp));
     cred_t *credp = crref();
     struct AFS_FLOCK flock;
-#if defined(POSIX_TEST_LOCK_CONFLICT_ARG)
-    struct file_lock conflict;
-#elif defined(POSIX_TEST_LOCK_RETURNS_CONFLICT)
-    struct file_lock *conflict;
-#endif
-    
     /* Convert to a lock format afs_lockctl understands. */
     memset((char *)&flock, 0, sizeof(flock));
     flock.l_type = flp->fl_type;
     flock.l_pid = flp->fl_pid;
     flock.l_whence = 0;
     flock.l_start = flp->fl_start;
-    if (flp->fl_end == OFFSET_MAX)
-	flock.l_len = 0; /* Lock to end of file */
-    else
-	flock.l_len = flp->fl_end - flp->fl_start + 1;
-
-    /* Safe because there are no large files, yet */
-#if defined(F_GETLK64) && (F_GETLK != F_GETLK64)
-    if (cmd == F_GETLK64)
-	cmd = F_GETLK;
-    else if (cmd == F_SETLK64)
-	cmd = F_SETLK;
-    else if (cmd == F_SETLKW64)
-	cmd = F_SETLKW;
-#endif /* F_GETLK64 && F_GETLK != F_GETLK64 */
-
-    AFS_GLOCK();
-    code = afs_convert_code(afs_lockctl(vcp, &flock, cmd, credp));
-    AFS_GUNLOCK();
-
-#ifdef AFS_LINUX24_ENV
-    if ((code == 0 || flp->fl_type == F_UNLCK) && 
-        (cmd == F_SETLK || cmd == F_SETLKW)) {
-# ifdef POSIX_LOCK_FILE_WAIT_ARG
-	code = posix_lock_file(fp, flp, 0);
-# else
-	flp->fl_flags &=~ FL_SLEEP;
-	code = posix_lock_file(fp, flp);
-# endif 
-	if (code && flp->fl_type != F_UNLCK) {
-	    struct AFS_FLOCK flock2;
-	    flock2 = flock;
-	    flock2.l_type = F_UNLCK;
-	    AFS_GLOCK();
-	    afs_lockctl(vcp, &flock2, F_SETLK, credp);
-	    AFS_GUNLOCK();
-	}
-    }
-    /* If lockctl says there are no conflicting locks, then also check with the
-     * kernel, as lockctl knows nothing about byte range locks
-     */
-    if (code == 0 && cmd == F_GETLK && flock.l_type == F_UNLCK) {
-# if defined(POSIX_TEST_LOCK_CONFLICT_ARG)
-        if (posix_test_lock(fp, flp, &conflict)) {
-            locks_copy_lock(flp, &conflict);
-            flp->fl_type = F_UNLCK;
-            crfree(credp);
-            return 0;
-        }
-# elif defined(POSIX_TEST_LOCK_RETURNS_CONFLICT)
-	if ((conflict = posix_test_lock(fp, flp))) {
-	    locks_copy_lock(flp, conflict);
-	    flp->fl_type = F_UNLCK;
-	    crfree(credp);
-	    return 0;
-	}
-# else
-        posix_test_lock(fp, flp);
-        /* If we found a lock in the kernel's structure, return it */
-        if (flp->fl_type != F_UNLCK) {
-            crfree(credp);
-            return 0;
-        }
-# endif
-    }
-    
-#endif
-    /* Convert flock back to Linux's file_lock */
-    flp->fl_type = flock.l_type;
-    flp->fl_pid = flock.l_pid;
-    flp->fl_start = flock.l_start;
-    if (flock.l_len == 0)
-	flp->fl_end = OFFSET_MAX; /* Lock to end of file */
-    else
-	flp->fl_end = flock.l_start + flock.l_len - 1;
-
-    crfree(credp);
-    return code;
-}
-
-#ifdef STRUCT_FILE_OPERATIONS_HAS_FLOCK
-static int
-afs_linux_flock(struct file *fp, int cmd, struct file_lock *flp) {
-    int code = 0;
-    struct vcache *vcp = VTOAFS(FILE_INODE(fp));
-    cred_t *credp = crref();
-    struct AFS_FLOCK flock;
-    /* Convert to a lock format afs_lockctl understands. */
-    memset((char *)&flock, 0, sizeof(flock));
-    flock.l_type = flp->fl_type;
-    flock.l_pid = flp->fl_pid;
-    flock.l_whence = 0;
-    flock.l_start = 0;
-    flock.l_len = 0;
+    flock.l_len = flp->fl_end - flp->fl_start;
 
     /* Safe because there are no large files, yet */
 #if defined(F_GETLK64) && (F_GETLK != F_GETLK64)
@@ -601,27 +463,16 @@ afs_linux_flock(struct file *fp, int cmd, struct file_lock *flp) {
     code = afs_lockctl(vcp, &flock, cmd, credp);
     AFS_GUNLOCK();
 
-    if ((code == 0 || flp->fl_type == F_UNLCK) && 
-        (cmd == F_SETLK || cmd == F_SETLKW)) {
-	flp->fl_flags &=~ FL_SLEEP;
-	code = flock_lock_file_wait(fp, flp);
-	if (code && flp->fl_type != F_UNLCK) {
-	    struct AFS_FLOCK flock2;
-	    flock2 = flock;
-	    flock2.l_type = F_UNLCK;
-	    AFS_GLOCK();
-	    afs_lockctl(vcp, &flock2, F_SETLK, credp);
-	    AFS_GUNLOCK();
-	}
-    }
     /* Convert flock back to Linux's file_lock */
     flp->fl_type = flock.l_type;
     flp->fl_pid = flock.l_pid;
+    flp->fl_start = flock.l_start;
+    flp->fl_end = flock.l_start + flock.l_len;
 
     crfree(credp);
-    return afs_convert_code(code);
+    return -code;
+
 }
-#endif
 
 /* afs_linux_flush
  * essentially the same as afs_fsync() but we need to get the return
@@ -629,33 +480,21 @@ afs_linux_flock(struct file *fp, int cmd, struct file_lock *flp) {
  * afs_StoreAllSegments() with AFS_LASTSTORE
  */
 static int
-#if defined(FOP_FLUSH_TAKES_FL_OWNER_T)
-afs_linux_flush(struct file *fp, fl_owner_t id)
-#else
 afs_linux_flush(struct file *fp)
-#endif
 {
     struct vrequest treq;
-    struct vcache *vcp;
-    cred_t *credp;
+    struct vcache *vcp = VTOAFS(FILE_INODE(fp));
+    cred_t *credp = crref();
     int code;
 
     AFS_GLOCK();
 
-    if ((fp->f_flags & O_ACCMODE) == O_RDONLY) { /* readers dont flush */
-	AFS_GUNLOCK();
-	return 0;
-    }
-
-    credp = crref();
-    vcp = VTOAFS(FILE_INODE(fp));
-
     code = afs_InitReq(&treq, credp);
     if (code)
 	goto out;
 
     ObtainSharedLock(&vcp->lock, 535);
-    if ((vcp->execsOrWriters > 0) && (file_count(fp) == 1)) {
+    if (vcp->execsOrWriters > 0) {
 	UpgradeSToWLock(&vcp->lock, 536);
 	code = afs_StoreAllSegments(vcp, &treq, AFS_SYNC | AFS_LASTSTORE);
 	ConvertWToSLock(&vcp->lock);
@@ -667,7 +506,7 @@ out:
     AFS_GUNLOCK();
 
     crfree(credp);
-    return afs_convert_code(code);
+    return -code;
 }
 
 #if !defined(AFS_LINUX24_ENV)
@@ -700,16 +539,11 @@ struct file_operations afs_dir_fops = {
 #endif
   .open =	afs_linux_open,
   .release =	afs_linux_release,
-  .llseek =	default_llseek,
 };
 
 struct file_operations afs_file_fops = {
   .read =	afs_linux_read,
   .write =	afs_linux_write,
-#ifdef GENERIC_FILE_AIO_READ
-  .aio_read =	generic_file_aio_read,
-  .aio_write =	generic_file_aio_write,
-#endif
 #ifdef HAVE_UNLOCKED_IOCTL
   .unlocked_ioctl = afs_unlocked_xioctl,
 #else
@@ -721,20 +555,12 @@ struct file_operations afs_file_fops = {
   .mmap =	afs_linux_mmap,
   .open =	afs_linux_open,
   .flush =	afs_linux_flush,
-#if defined(AFS_LINUX26_ENV) && defined(STRUCT_FILE_OPERATIONS_HAS_SENDFILE)
+#ifdef AFS_LINUX26_ENV
   .sendfile =   generic_file_sendfile,
-#endif
-#if defined(AFS_LINUX26_ENV) && defined(STRUCT_FILE_OPERATIONS_HAS_SPLICE)
-  .splice_write = generic_file_splice_write,
-  .splice_read = generic_file_splice_read,
 #endif
   .release =	afs_linux_release,
   .fsync =	afs_linux_fsync,
   .lock =	afs_linux_lock,
-#ifdef STRUCT_FILE_OPERATIONS_HAS_FLOCK
-  .flock =	afs_linux_flock,
-#endif
-  .llseek = 	default_llseek,
 };
 
 
@@ -818,7 +644,7 @@ afs_linux_revalidate(struct dentry *dp)
     credp = crref();
     code = afs_getattr(vcp, &vattr, credp);
     if (!code)
-        afs_fill_inode(AFSTOV(vcp), &vattr);
+        vattr2inode(AFSTOV(vcp), &vattr);
 
     AFS_GUNLOCK();
 #ifdef AFS_LINUX24_ENV
@@ -826,7 +652,7 @@ afs_linux_revalidate(struct dentry *dp)
 #endif
     crfree(credp);
 
-    return afs_convert_code(code);
+    return -code;
 }
 
 #if defined(AFS_LINUX26_ENV)
@@ -863,20 +689,12 @@ afs_linux_dentry_revalidate(struct dentry *dp)
     cred_t *credp = NULL;
     struct vcache *vcp, *pvcp, *tvc = NULL;
     int valid;
-    struct afs_fakestat_state fakestate;
 
-#ifdef LOOKUP_RCU
-    /* We don't support RCU path walking */
-    if (nd->flags & LOOKUP_RCU)
-       return -ECHILD;
-#endif
 #ifdef AFS_LINUX24_ENV
     lock_kernel();
 #endif
     AFS_GLOCK();
 
-    afs_InitFakeStat(&fakestate);
-
     if (dp->d_inode) {
 
 	vcp = VTOAFS(dp->d_inode);
@@ -885,34 +703,8 @@ afs_linux_dentry_revalidate(struct dentry *dp)
 	if (vcp == afs_globalVp)
 	    goto good_dentry;
 
-	if (vcp->mvstat == 1) {         /* mount point */
-	    if (vcp->mvid && (vcp->states & CMValid)) {
-		int tryEvalOnly = 0;
-		int code = 0;
-		struct vrequest treq;
-
-		credp = crref();
-		code = afs_InitReq(&treq, credp);
-		if (
-#ifdef AFS_DARWIN_ENV
-		    (strcmp(dp->d_name.name, ".DS_Store") == 0) ||
-		    (strcmp(dp->d_name.name, "Contents") == 0) ||
-#endif
-		    (strcmp(dp->d_name.name, ".directory") == 0)) {
-		    tryEvalOnly = 1;
-		}
-		if (tryEvalOnly)
-		    code = afs_TryEvalFakeStat(&vcp, &fakestate, &treq);
-		else
-		    code = afs_EvalFakeStat(&vcp, &fakestate, &treq);
-		if ((tryEvalOnly && vcp->mvstat == 1) || code) {
-		    /* a mount point, not yet replaced by its directory */
-		    goto bad_dentry;
-		}
-	    }
-	} else
-	    if (*dp->d_name.name != '/' && vcp->mvstat == 2) /* root vnode */
-		check_bad_parent(dp);	/* check and correct mvid */
+	if (*dp->d_name.name != '/' && vcp->mvstat == 2)	/* root vnode */
+	    check_bad_parent(dp);	/* check and correct mvid */
 
 #ifdef notdef
 	/* If the last looker changes, we should make sure the current
@@ -972,7 +764,6 @@ afs_linux_dentry_revalidate(struct dentry *dp)
     /* Clean up */
     if (tvc)
 	afs_PutVCache(tvc);
-    afs_PutFakeStat(&fakestate);
     AFS_GUNLOCK();
     if (credp)
 	crfree(credp);
@@ -987,10 +778,7 @@ afs_linux_dentry_revalidate(struct dentry *dp)
     return valid;
 
   bad_dentry:
-    if (have_submounts(dp))
-	valid = 1;
-    else 
-	valid = 0;
+    valid = 0;
     goto done;
 }
 
@@ -1000,17 +788,9 @@ afs_dentry_iput(struct dentry *dp, struct inode *ip)
     struct vcache *vcp = VTOAFS(ip);
 
     AFS_GLOCK();
-    (void) afs_InactiveVCache(vcp, NULL);
+    if (vcp->states & CUnlinked)
+	(void) afs_InactiveVCache(vcp, NULL);
     AFS_GUNLOCK();
-#ifdef DCACHE_NFSFS_RENAMED
-#ifdef AFS_LINUX26_ENV
-    spin_lock(&dp->d_lock);
-#endif
-    dp->d_flags &= ~DCACHE_NFSFS_RENAMED;   
-#ifdef AFS_LINUX26_ENV
-    spin_unlock(&dp->d_lock);
-#endif
-#endif
 
     iput(ip);
 }
@@ -1072,10 +852,7 @@ afs_linux_create(struct inode *dip, struct dentry *dp, int mode)
 
 	afs_getattr(vcp, &vattr, credp);
 	afs_fill_inode(ip, &vattr);
-	insert_inode_hash(ip);
-#if !defined(STRUCT_SUPER_BLOCK_HAS_S_D_OP)
 	dp->d_op = &afs_dentry_operations;
-#endif
 	dp->d_time = hgetlo(VTOAFS(dip)->m.DataVersion);
 	d_instantiate(dp, ip);
     }
@@ -1085,7 +862,7 @@ afs_linux_create(struct inode *dip, struct dentry *dp, int mode)
     unlock_kernel();
 #endif
     crfree(credp);
-    return afs_convert_code(code);
+    return -code;
 }
 
 /* afs_linux_lookup */
@@ -1120,20 +897,8 @@ afs_linux_lookup(struct inode *dip, struct dentry *dp)
 	ip = AFSTOV(vcp);
 	afs_getattr(vcp, &vattr, credp);
 	afs_fill_inode(ip, &vattr);
-	if (
-#ifdef HAVE_KERNEL_HLIST_UNHASHED
-	    hlist_unhashed(&ip->i_hash)
-#elif defined(AFS_LINUX26_ENV)
-	    ip->i_hash.pprev == NULL
-#else
-	    ip->i_hash.prev == NULL
-#endif
-	    )
-	    insert_inode_hash(ip);
     }
-#if !defined(STRUCT_SUPER_BLOCK_HAS_S_D_OP)
     dp->d_op = &afs_dentry_operations;
-#endif
     dp->d_time = hgetlo(VTOAFS(dip)->m.DataVersion);
     AFS_GUNLOCK();
 
@@ -1150,7 +915,6 @@ afs_linux_lookup(struct inode *dip, struct dentry *dp)
 #if defined(AFS_LINUX26_ENV)
 		unlock_kernel();
 #endif
-		crfree(credp);
 		return alias;
 	    }
 	}
@@ -1170,11 +934,11 @@ afs_linux_lookup(struct inode *dip, struct dentry *dp)
     if (code == ENOENT)
 	return ERR_PTR(0);
     else
-	return ERR_PTR(afs_convert_code(code));
+	return ERR_PTR(-code);
 #else
     if (code == ENOENT)
 	code = 0;
-    return afs_convert_code(code);
+    return -code;
 #endif
 }
 
@@ -1196,7 +960,7 @@ afs_linux_link(struct dentry *olddp, struct inode *dip, struct dentry *newdp)
 
     AFS_GUNLOCK();
     crfree(credp);
-    return afs_convert_code(code);
+    return -code;
 }
 
 static int
@@ -1243,17 +1007,6 @@ afs_linux_unlink(struct inode *dip, struct dentry *dp)
             }
             tvc->uncred = credp;
 	    tvc->states |= CUnlinked;
-#ifdef DCACHE_NFSFS_RENAMED
-#ifdef AFS_LINUX26_ENV
-	    spin_lock(&dp->d_lock);
-#endif
-	    dp->d_flags |= DCACHE_NFSFS_RENAMED;   
-#ifdef AFS_LINUX26_ENV
-	    spin_unlock(&dp->d_lock);
-#endif
-#endif
-	} else {
-	    osi_FreeSmallSpace(__name);	
 	}
 	AFS_GUNLOCK();
 
@@ -1276,7 +1029,7 @@ out:
     unlock_kernel();
 #endif
     crfree(credp);
-    return afs_convert_code(code);
+    return -code;
 }
 
 
@@ -1298,7 +1051,7 @@ afs_linux_symlink(struct inode *dip, struct dentry *dp, const char *target)
     code = afs_symlink(VTOAFS(dip), name, &vattr, target, credp);
     AFS_GUNLOCK();
     crfree(credp);
-    return afs_convert_code(code);
+    return -code;
 }
 
 static int
@@ -1325,9 +1078,7 @@ afs_linux_mkdir(struct inode *dip, struct dentry *dp, int mode)
 	afs_getattr(tvcp, &vattr, credp);
 	afs_fill_inode(ip, &vattr);
 
-#if !defined(STRUCT_SUPER_BLOCK_HAS_S_D_OP)
 	dp->d_op = &afs_dentry_operations;
-#endif
 	dp->d_time = hgetlo(VTOAFS(dip)->m.DataVersion);
 	d_instantiate(dp, ip);
     }
@@ -1337,7 +1088,7 @@ afs_linux_mkdir(struct inode *dip, struct dentry *dp, int mode)
     unlock_kernel();
 #endif
     crfree(credp);
-    return afs_convert_code(code);
+    return -code;
 }
 
 static int
@@ -1366,7 +1117,7 @@ afs_linux_rmdir(struct inode *dip, struct dentry *dp)
     }
 
     crfree(credp);
-    return afs_convert_code(code);
+    return -code;
 }
 
 
@@ -1383,7 +1134,13 @@ afs_linux_rename(struct inode *oldip, struct dentry *olddp,
 #if defined(AFS_LINUX26_ENV)
     /* Prevent any new references during rename operation. */
     lock_kernel();
-
+#endif
+    /* Remove old and new entries from name hash. New one will change below.
+     * While it's optimal to catch failures and re-insert newdp into hash,
+     * it's also error prone and in that case we're already dealing with error
+     * cases. Let another lookup put things right, if need be.
+     */
+#if defined(AFS_LINUX26_ENV)
     if (!d_unhashed(newdp)) {
 	d_drop(newdp);
 	rehash = newdp;
@@ -1396,26 +1153,14 @@ afs_linux_rename(struct inode *oldip, struct dentry *olddp,
 #endif
 
 #if defined(AFS_LINUX24_ENV)
-#if defined(D_COUNT_INT)
-    spin_lock(&olddp->d_lock);
-    if (olddp->d_count > 1) {
-	spin_unlock(&olddp->d_lock);
-	shrink_dcache_parent(olddp);
-    } else
-	spin_unlock(&olddp->d_lock);
-#else
     if (atomic_read(&olddp->d_count) > 1)
 	shrink_dcache_parent(olddp);
-#endif
 #endif
 
     AFS_GLOCK();
     code = afs_rename(VTOAFS(oldip), oldname, VTOAFS(newip), newname, credp);
     AFS_GUNLOCK();
 
-    if (!code)
-	olddp->d_time = 0;      /* force to revalidate */
-
     if (rehash)
 	d_rehash(rehash);
 
@@ -1424,7 +1169,7 @@ afs_linux_rename(struct inode *oldip, struct dentry *olddp,
 #endif
 
     crfree(credp);
-    return afs_convert_code(code);
+    return -code;
 }
 
 
@@ -1447,10 +1192,10 @@ afs_linux_ireadlink(struct inode *ip, char *target, int maxlen, uio_seg_t seg)
     if (!code)
 	return maxlen - tuio.uio_resid;
     else
-	return afs_convert_code(code);
+	return -code;
 }
 
-#if !defined(USABLE_KERNEL_PAGE_SYMLINK_CACHE)
+#if !defined(AFS_LINUX24_ENV)
 /* afs_linux_readlink 
  * Fill target (which is in user space) with contents of symlink.
  */
@@ -1470,36 +1215,6 @@ afs_linux_readlink(struct dentry *dp, char *target, int maxlen)
 /* afs_linux_follow_link
  * a file system dependent link following routine.
  */
-#if defined(AFS_LINUX24_ENV)
-static int afs_linux_follow_link(struct dentry *dentry, struct nameidata *nd)
-{
-    int code;
-    char *name;
-
-    name = osi_Alloc(PATH_MAX);
-    if (!name) {
-	return -EIO;
-    }
-
-    AFS_GLOCK();
-    code = afs_linux_ireadlink(dentry->d_inode, name, PATH_MAX - 1, AFS_UIOSYS);
-    AFS_GUNLOCK();
-
-    if (code < 0) {
-	goto out;
-    }
-
-    name[code] = '\0';
-    code = vfs_follow_link(nd, name);
-
-out:
-    osi_Free(name, PATH_MAX);
-
-    return code;
-}
-
-#else /* !defined(AFS_LINUX24_ENV) */
-
 static struct dentry *
 afs_linux_follow_link(struct dentry *dp, struct dentry *basep,
 		      unsigned int follow)
@@ -1522,10 +1237,7 @@ afs_linux_follow_link(struct dentry *dp, struct dentry *basep,
 
     if (code < 0) {
 	dput(basep);
-	if (code < -MAX_ERRNO)
-	    res = ERR_PTR(-EIO);
-	else
-	    res = ERR_PTR(code);
+	res = ERR_PTR(code);
     } else {
 	name[code] = '\0';
 	res = lookup_dentry(name, basep, follow);
@@ -1536,8 +1248,7 @@ afs_linux_follow_link(struct dentry *dp, struct dentry *basep,
     AFS_GUNLOCK();
     return res;
 }
-#endif /* AFS_LINUX24_ENV */
-#endif /* USABLE_KERNEL_PAGE_SYMLINK_CACHE */
+#endif
 
 /* afs_linux_readpage
  * all reads come through here. A strategy-like read call.
@@ -1570,7 +1281,7 @@ afs_linux_readpage(struct file *fp, struct page *pp)
     clear_bit(PG_error, &pp->flags);
 #endif
 
-    setup_uio(&tuio, &iovec, (char *)address, offset, PAGE_SIZE, UIO_READ,
+    setup_uio(&tuio, &iovec, (char *)address, offset, PAGESIZE, UIO_READ,
 	      AFS_UIOSYS);
 #ifdef AFS_LINUX24_ENV
     lock_kernel();
@@ -1588,7 +1299,7 @@ afs_linux_readpage(struct file *fp, struct page *pp)
 
     if (!code) {
 	if (tuio.uio_resid)	/* zero remainder of page */
-	    memset((void *)(address + (PAGE_SIZE - tuio.uio_resid)), 0,
+	    memset((void *)(address + (PAGESIZE - tuio.uio_resid)), 0,
 		   tuio.uio_resid);
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,0)
 	flush_dcache_page(pp);
@@ -1626,7 +1337,7 @@ afs_linux_readpage(struct file *fp, struct page *pp)
     }
 
     crfree(credp);
-    return afs_convert_code(code);
+    return -code;
 }
 
 
@@ -1654,41 +1365,22 @@ afs_linux_writepage_sync(struct inode *ip, struct page *pp,
 	       ICL_TYPE_POINTER, pp, ICL_TYPE_INT32, page_count(pp),
 	       ICL_TYPE_INT32, 99999);
 
-    ObtainWriteLock(&vcp->lock, 532);
-    if (vcp->states & CPageWrite) {
-	ReleaseWriteLock(&vcp->lock);
-	AFS_GUNLOCK();
-	unlock_kernel();
-	crfree(credp);
-	kunmap(pp);
-#ifdef AFS_LINUX26_ENV
-	return WRITEPAGE_ACTIVATE;
-#else
-	/* should mark it dirty? */
-	return(0); 
-#endif
-    }
-    vcp->states |= CPageWrite;
-    ReleaseWriteLock(&vcp->lock);
-
     setup_uio(&tuio, &iovec, buffer, base, count, UIO_WRITE, AFS_UIOSYS);
 
     code = afs_write(vcp, &tuio, f_flags, credp, 0);
 
-    i_size_write(ip, vcp->m.Length);
+    ip->i_size = vcp->m.Length;
     ip->i_blocks = ((vcp->m.Length + 1023) >> 10) << 1;
 
-    ObtainWriteLock(&vcp->lock, 533);
     if (!code) {
 	struct vrequest treq;
 
+	ObtainWriteLock(&vcp->lock, 533);
 	if (!afs_InitReq(&treq, credp))
 	    code = afs_DoPartialWrite(vcp, &treq);
+	ReleaseWriteLock(&vcp->lock);
     }
-    code = code ? afs_convert_code(code) : count - tuio.uio_resid;
-
-    vcp->states &= ~CPageWrite;
-    ReleaseWriteLock(&vcp->lock);
+    code = code ? -code : count - tuio.uio_resid;
 
     afs_Trace4(afs_iclSetp, CM_TRACE_UPDATEPAGE, ICL_TYPE_POINTER, vcp,
 	       ICL_TYPE_POINTER, pp, ICL_TYPE_INT32, page_count(pp),
@@ -1702,6 +1394,7 @@ afs_linux_writepage_sync(struct inode *ip, struct page *pp,
     return code;
 }
 
+
 static int
 #ifdef AOP_WRITEPAGE_TAKES_WRITEBACK_CONTROL
 afs_linux_writepage(struct page *pp, struct writeback_control *wbc)
@@ -1716,8 +1409,13 @@ afs_linux_writepage(struct page *pp)
     long status;
 
 #if defined(AFS_LINUX26_ENV)
-    if (PageReclaim(pp))
+    if (PageReclaim(pp)) {
+# if defined(WRITEPAGE_ACTIVATE)
 	return WRITEPAGE_ACTIVATE;
+# else 
+	return AOP_WRITEPAGE_ACTIVATE;
+# endif
+    }
 #else
     if (PageLaunder(pp)) {
 	return(fail_writepage(pp));
@@ -1725,23 +1423,20 @@ afs_linux_writepage(struct page *pp)
 #endif
 
     inode = (struct inode *)mapping->host;
-    end_index = i_size_read(inode) >> PAGE_CACHE_SHIFT;
+    end_index = inode->i_size >> PAGE_CACHE_SHIFT;
 
     /* easy case */
     if (pp->index < end_index)
 	goto do_it;
     /* things got complicated... */
-    offset = i_size_read(inode) & (PAGE_CACHE_SIZE - 1);
+    offset = inode->i_size & (PAGE_CACHE_SIZE - 1);
     /* OK, are we completely out? */
     if (pp->index >= end_index + 1 || !offset)
 	return -EIO;
   do_it:
     status = afs_linux_writepage_sync(inode, pp, 0, offset);
     SetPageUptodate(pp);
-#if defined(AFS_LINUX26_ENV)
-    if ( status != WRITEPAGE_ACTIVATE )
-#endif
-	UnlockPage(pp);
+    UnlockPage(pp);
     if (status == offset)
 	return 0;
     else
@@ -1777,7 +1472,7 @@ afs_linux_updatepage(struct file *fp, struct page *pp, unsigned long offset,
 
     code = afs_write(vcp, &tuio, fp->f_flags, credp, 0);
 
-    i_size_write(ip, vcp->m.Length);
+    ip->i_size = vcp->m.Length;
     ip->i_blocks = ((vcp->m.Length + 1023) >> 10) << 1;
 
     if (!code) {
@@ -1790,7 +1485,7 @@ afs_linux_updatepage(struct file *fp, struct page *pp, unsigned long offset,
 	ReleaseWriteLock(&vcp->lock);
     }
 
-    code = code ? afs_convert_code(code) : count - tuio.uio_resid;
+    code = code ? -code : count - tuio.uio_resid;
     afs_Trace4(afs_iclSetp, CM_TRACE_UPDATEPAGE, ICL_TYPE_POINTER, vcp,
 	       ICL_TYPE_POINTER, pp, ICL_TYPE_INT32, page_count(pp),
 	       ICL_TYPE_INT32, code);
@@ -1807,25 +1502,16 @@ afs_linux_updatepage(struct file *fp, struct page *pp, unsigned long offset,
  * Check access rights - returns error if can't check or permission denied.
  */
 static int
-#if defined(IOP_PERMISSION_TAKES_FLAGS)
-afs_linux_permission(struct inode *ip, int mode, unsigned int flags)
-#elif defined(IOP_PERMISSION_TAKES_NAMEIDATA)
+#ifdef IOP_PERMISSION_TAKES_NAMEIDATA
 afs_linux_permission(struct inode *ip, int mode, struct nameidata *nd)
 #else
 afs_linux_permission(struct inode *ip, int mode)
 #endif
 {
     int code;
-    cred_t *credp;
+    cred_t *credp = crref();
     int tmp = 0;
 
-#if defined(IOP_PERMISSION_TAKES_FLAGS)
-    /* We don't support RCU path walking */
-    if (flags & IPERM_FLAG_RCU)
-       return -ECHILD;
-#endif
-
-    credp = crref();
     AFS_GLOCK();
     if (mode & MAY_EXEC)
 	tmp |= VEXEC;
@@ -1837,10 +1523,10 @@ afs_linux_permission(struct inode *ip, int mode)
 
     AFS_GUNLOCK();
     crfree(credp);
-    return afs_convert_code(code);
+    return -code;
 }
 
-#if defined(AFS_LINUX24_ENV) && !(LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,28) && defined(HAVE_WRITE_BEGIN))
+#if defined(AFS_LINUX24_ENV)
 static int
 afs_linux_commit_write(struct file *file, struct page *page, unsigned offset,
 		       unsigned to)
@@ -1852,8 +1538,6 @@ afs_linux_commit_write(struct file *file, struct page *page, unsigned offset,
 #if !defined(AFS_LINUX26_ENV)
     kunmap(page);
 #endif
-    if (code == WRITEPAGE_ACTIVATE)
-	code = -EIO;
 
     return code;
 }
@@ -1869,43 +1553,8 @@ afs_linux_prepare_write(struct file *file, struct page *page, unsigned from,
 #endif
     return 0;
 }
-#endif
 
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,28) && defined(HAVE_WRITE_BEGIN)
-static int
-afs_linux_write_end(struct file *file, struct address_space *mapping,
-                                loff_t pos, unsigned len, unsigned copied,
-                                struct page *page, void *fsdata)
-{
-    int code;
-    pgoff_t index = pos >> PAGE_CACHE_SHIFT;
-    unsigned from = pos & (PAGE_CACHE_SIZE - 1);
-
-    code = afs_linux_writepage_sync(file->f_dentry->d_inode, page,
-				    from, copied);
-    unlock_page(page);
-    page_cache_release(page);
-    if (code == WRITEPAGE_ACTIVATE)
-	code = -EIO;
-    return code;
-}
-
-static int
-afs_linux_write_begin(struct file *file, struct address_space *mapping,
-                                loff_t pos, unsigned len, unsigned flags,
-                                struct page **pagep, void **fsdata)
-{
-    struct page *page;
-    pgoff_t index = pos >> PAGE_CACHE_SHIFT;
-#if defined(HAVE_GRAB_CACHE_PAGE_WRITE_BEGIN)
-    page = grab_cache_page_write_begin(mapping, index, flags);
-#else
-    page = __grab_cache_page(mapping, index);
-#endif
-    *pagep = page;
-
-    return 0;
-}
+extern int afs_notify_change(struct dentry *dp, struct iattr *iattrp);
 #endif
 
 static struct inode_operations afs_file_iops = {
@@ -1929,13 +1578,8 @@ static struct inode_operations afs_file_iops = {
 static struct address_space_operations afs_file_aops = {
   .readpage =		afs_linux_readpage,
   .writepage =		afs_linux_writepage,
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,28) && defined(HAVE_WRITE_BEGIN)
-  .write_begin =        afs_linux_write_begin,
-  .write_end =          afs_linux_write_end,
-#else
-  .commit_write =       afs_linux_commit_write,
-  .prepare_write =      afs_linux_prepare_write,
-#endif
+  .commit_write =	afs_linux_commit_write,
+  .prepare_write =	afs_linux_prepare_write,
 };
 #endif
 
@@ -1969,7 +1613,7 @@ static struct inode_operations afs_dir_iops = {
 /* We really need a separate symlink set of ops, since do_follow_link()
  * determines if it _is_ a link by checking if the follow_link op is set.
  */
-#if defined(USABLE_KERNEL_PAGE_SYMLINK_CACHE)
+#if defined(AFS_LINUX24_ENV)
 static int
 afs_symlink_filler(struct file *file, struct page *page)
 {
@@ -2004,10 +1648,10 @@ afs_symlink_filler(struct file *file, struct page *page)
 static struct address_space_operations afs_symlink_aops = {
   .readpage =	afs_symlink_filler
 };
-#endif	/* USABLE_KERNEL_PAGE_SYMLINK_CACHE */
+#endif
 
 static struct inode_operations afs_symlink_iops = {
-#if defined(USABLE_KERNEL_PAGE_SYMLINK_CACHE)
+#if defined(AFS_LINUX24_ENV)
   .readlink = 		page_readlink,
 #if defined(HAVE_KERNEL_PAGE_FOLLOW_LINK)
   .follow_link =	page_follow_link,
@@ -2015,17 +1659,13 @@ static struct inode_operations afs_symlink_iops = {
   .follow_link =	page_follow_link_light,
   .put_link =           page_put_link,
 #endif
-#else /* !defined(USABLE_KERNEL_PAGE_SYMLINK_CACHE) */
+  .setattr =		afs_notify_change,
+#else
   .readlink = 		afs_linux_readlink,
   .follow_link =	afs_linux_follow_link,
-#if !defined(AFS_LINUX24_ENV)
   .permission =		afs_linux_permission,
   .revalidate =		afs_linux_revalidate,
 #endif
-#endif /* USABLE_KERNEL_PAGE_SYMLINK_CACHE */
-#if defined(AFS_LINUX24_ENV)
-  .setattr =		afs_notify_change,
-#endif
 };
 
 void
@@ -2035,9 +1675,6 @@ afs_fill_inode(struct inode *ip, struct vattr *vattr)
     if (vattr)
 	vattr2inode(ip, vattr);
 
-#if defined(AFS_LINUX26_ENV)
-    ip->i_mapping->backing_dev_info = afs_backing_dev_info;
-#endif
 /* Reset ops if symlink or directory. */
     if (S_ISREG(ip->i_mode)) {
 	ip->i_op = &afs_file_iops;
@@ -2054,10 +1691,11 @@ afs_fill_inode(struct inode *ip, struct vattr *vattr)
 
     } else if (S_ISLNK(ip->i_mode)) {
 	ip->i_op = &afs_symlink_iops;
-#if defined(USABLE_KERNEL_PAGE_SYMLINK_CACHE)
+#if defined(AFS_LINUX24_ENV)
 	ip->i_data.a_ops = &afs_symlink_aops;
 	ip->i_mapping = &ip->i_data;
 #endif
     }
 
+    /* insert_inode_hash(ip);	-- this would make iget() work (if we used it) */
 }
