@@ -10,6 +10,8 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
+RCSID
+    ("$Header: /cvs/openafs/src/bozo/bosserver.c,v 1.30 2006/02/01 16:09:15 shadow Exp $");
 
 #include <afs/stds.h>
 #include <sys/types.h>
@@ -19,7 +21,6 @@
 #include <string.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <stdarg.h>
 #ifdef AFS_NT40_ENV
 #include <winsock2.h>
 #include <direct.h>
@@ -36,7 +37,6 @@
 #include <rx/rx_globals.h>
 #include "bosint.h"
 #include "bnode.h"
-#include "bosprototypes.h"
 #include <afs/auth.h>
 #include <afs/keys.h>
 #include <afs/ktime.h>
@@ -47,11 +47,14 @@
 #include <afs/afs_args.h>
 #endif
 
+
 #define BOZO_LWP_STACKSIZE	16000
 extern int BOZO_ExecuteRequest();
 extern int RXSTATS_ExecuteRequest();
 extern struct bnode_ops fsbnode_ops, ezbnode_ops, cronbnode_ops;
 
+void bozo_Log();
+
 struct afsconf_dir *bozo_confdir = 0;	/* bozo configuration dir */
 static char *bozo_pid;
 struct rx_securityClass *bozo_rxsc[3];
@@ -68,12 +71,6 @@ static afs_int32 nextDay;
 
 struct ktime bozo_nextRestartKT, bozo_nextDayKT;
 int bozo_newKTs;
-int rxBind = 0;
-int rxkadDisableDotCheck = 0;
-
-#define ADDRSPERSITE 16         /* Same global is in rx/rx_user.c */
-afs_uint32 SHostAddrs[ADDRSPERSITE];
-
 #ifdef BOS_RESTRICTED_MODE
 int bozo_isrestricted = 0;
 int bozo_restdisable = 0;
@@ -417,7 +414,7 @@ ReadBozoFile(char *aname)
 	code =
 	    bnode_Create(typep, instp, &tb, parms[0], parms[1], parms[2],
 			 parms[3], parms[4], notifier,
-			 goal ? BSTAT_NORMAL : BSTAT_SHUTDOWN, 0);
+			 goal ? BSTAT_NORMAL : BSTAT_SHUTDOWN);
 	if (code)
 	    goto fail;
 
@@ -512,8 +509,8 @@ bdrestart(register struct bnode *abnode, char *arock)
 
 #define	BOZO_MINSKIP 3600	/* minimum to advance clock */
 /* lwp to handle system restarts */
-static void *
-BozoDaemon(void *unused)
+static int
+BozoDaemon()
 {
     register afs_int32 now;
 
@@ -548,7 +545,6 @@ BozoDaemon(void *unused)
 	    bnode_ApplyInstance(bdrestart, 0);
 	}
     }
-    return NULL;
 }
 
 #ifdef AFS_AIX32_ENV
@@ -722,8 +718,6 @@ main(int argc, char **argv, char **envp)
     struct ktc_encryptionKey tkey;
     int i;
     char namebuf[AFSDIR_PATH_MAX];
-    int rxMaxMTU = -1;
-    afs_uint32 host = htonl(INADDR_ANY);
 #ifndef AFS_NT40_ENV
     int nofork = 0;
     struct stat sb;
@@ -818,30 +812,35 @@ main(int argc, char **argv, char **envp)
 	    bozo_isrestricted = 1;
 	}
 #endif
-	else if (strcmp(argv[code], "-rxbind") == 0) {
-	    rxBind = 1;
-	}
-	else if (strcmp(argv[code], "-allow-dotted-principals") == 0) {
-	    rxkadDisableDotCheck = 1;
-	}
-	else if (!strcmp(argv[code], "-rxmaxmtu")) {
-	    if ((code + 1) >= argc) {
-		fprintf(stderr, "missing argument for -rxmaxmtu\n"); 
-		exit(1); 
-	    }
-	    rxMaxMTU = atoi(argv[++code]);
-	    if ((rxMaxMTU < RX_MIN_PACKET_SIZE) || 
-		(rxMaxMTU > RX_MAX_PACKET_DATA_SIZE)) {
-		printf("rxMaxMTU %d invalid; must be between %d-%d\n",
-			rxMaxMTU, RX_MIN_PACKET_SIZE, 
-			RX_MAX_PACKET_DATA_SIZE);
-		exit(1);
-	    }
-	}
 	else if (strcmp(argv[code], "-auditlog") == 0) {
+	    int tempfd, flags;
+	    FILE *auditout;
+	    char oldName[MAXPATHLEN];
 	    char *fileName = argv[++code];
 
-            osi_audit_file(fileName);
+#ifndef AFS_NT40_ENV
+	    struct stat statbuf;
+	    
+	    if ((lstat(fileName, &statbuf) == 0) 
+		&& (S_ISFIFO(statbuf.st_mode))) {
+		flags = O_WRONLY | O_NONBLOCK;
+	    } else 
+#endif
+	    {
+		strcpy(oldName, fileName);
+		strcat(oldName, ".old");
+		renamefile(fileName, oldName);
+		flags = O_WRONLY | O_TRUNC | O_CREAT;
+	    }
+	    tempfd = open(fileName, flags, 0666);
+	    if (tempfd > -1) {
+		auditout = fdopen(tempfd, "a");
+		if (auditout) {
+		    osi_audit_file(auditout);
+		} else
+		    printf("Warning: auditlog %s not writable, ignored.\n", fileName);
+	    } else
+		printf("Warning: auditlog %s not writable, ignored.\n", fileName);
 	}
 	else {
 
@@ -850,14 +849,12 @@ main(int argc, char **argv, char **envp)
 #ifndef AFS_NT40_ENV
 	    printf("Usage: bosserver [-noauth] [-log] "
 		   "[-auditlog <log path>] "
-		   "[-rxmaxmtu <bytes>] [-rxbind] [-allow-dotted-principals]"
 		   "[-syslog[=FACILITY]] "
 		   "[-enable_peer_stats] [-enable_process_stats] "
 		   "[-nofork] " "[-help]\n");
 #else
 	    printf("Usage: bosserver [-noauth] [-log] "
 		   "[-auditlog <log path>] "
-		   "[-rxmaxmtu <bytes>] [-rxbind] [-allow-dotted-principals]"
 		   "[-enable_peer_stats] [-enable_process_stats] "
 		   "[-help]\n");
 #endif
@@ -1005,38 +1002,11 @@ main(int argc, char **argv, char **envp)
     bozo_rxsc[1] = (struct rx_securityClass *)0;
     bozo_rxsc[2] =
 	rxkad_NewServerSecurityObject(0, tdir, afsconf_GetKey, NULL);
-#ifdef USE_RXKAD_KEYTAB
-    if (rxkad_InitKeytabDecrypt(AFSDIR_SERVER_RXKAD_KEYTAB_FILEPATH) == 0)
-        rxkad_BindKeytabDecrypt(bozo_rxsc[2]);
-#endif
 
     /* Disable jumbograms */
     rx_SetNoJumbo();
 
-    if (rxMaxMTU != -1) {
-	rx_SetMaxMTU(rxMaxMTU);
-    }
-
-    if (rxBind) {
-	afs_int32 ccode;
-#ifndef AFS_NT40_ENV
-        if (AFSDIR_SERVER_NETRESTRICT_FILEPATH || 
-            AFSDIR_SERVER_NETINFO_FILEPATH) {
-            char reason[1024];
-            ccode = parseNetFiles(SHostAddrs, NULL, NULL,
-                                           ADDRSPERSITE, reason,
-                                           AFSDIR_SERVER_NETINFO_FILEPATH,
-                                           AFSDIR_SERVER_NETRESTRICT_FILEPATH);
-        } else 
-#endif	
-	{
-            ccode = rx_getAllAddr(SHostAddrs, ADDRSPERSITE);
-        }
-        if (ccode == 1) 
-            host = SHostAddrs[0];
-    }
-
-    tservice = rx_NewServiceHost(host,  /* port */ 0, /* service id */ 1,
+    tservice = rx_NewService( /* port */ 0, /* service id */ 1,
 			     /*service name */ "bozo",
 			     /* security classes */
 			     bozo_rxsc,
@@ -1044,33 +1014,24 @@ main(int argc, char **argv, char **envp)
     rx_SetMinProcs(tservice, 2);
     rx_SetMaxProcs(tservice, 4);
     rx_SetStackSize(tservice, BOZO_LWP_STACKSIZE);	/* so gethostbyname works (in cell stuff) */
-    if (rxkadDisableDotCheck) {
-        rx_SetSecurityConfiguration(tservice, RXS_CONFIG_FLAGS,
-                                    (void *)RXS_CONFIG_FLAGS_DISABLE_DOTCHECK, 
-                                    NULL);
-    }
 
     tservice =
-	rx_NewServiceHost(host, 0, RX_STATS_SERVICE_ID, "rpcstats", bozo_rxsc,
-			  3, RXSTATS_ExecuteRequest);
+	rx_NewService(0, RX_STATS_SERVICE_ID, "rpcstats", bozo_rxsc, 3,
+		      RXSTATS_ExecuteRequest);
     rx_SetMinProcs(tservice, 2);
     rx_SetMaxProcs(tservice, 4);
     rx_StartServer(1);		/* donate this process */
-    return 0;
 }
 
 void
-bozo_Log(char *format, ...)
+bozo_Log(char *a, char *b, char *c, char *d, char *e, char *f)
 {
-    char tdate[26];
+    char tdate[27];
     time_t myTime;
-    va_list ap;
-
-    va_start(ap, format);
 
     if (DoSyslog) {
 #ifndef AFS_NT40_ENV
-	vsyslog(LOG_INFO, format, ap);
+	syslog(LOG_INFO, a, b, c, d, e, f);
 #endif
     } else {
 	myTime = time(0);
@@ -1081,20 +1042,17 @@ bozo_Log(char *format, ...)
 
 	bozo_logFile = fopen(AFSDIR_SERVER_BOZLOG_FILEPATH, "a");
 	if (bozo_logFile == NULL) {
-	    printf("bosserver: WARNING: problem with %s",
+	    printf("bosserver: WARNING: problem with %s\n",
 		   AFSDIR_SERVER_BOZLOG_FILEPATH);
+	    printf("%s ", tdate);
+	    printf(a, b, c, d, e, f);
 	    fflush(stdout);
-	}
-
-	if (bozo_logFile) {
+	} else {
 	    fprintf(bozo_logFile, "%s ", tdate);
-	    vfprintf(bozo_logFile, format, ap);
-	    fflush(bozo_logFile);
+	    fprintf(bozo_logFile, a, b, c, d, e, f);
+
 	    /* close so rm BosLog works */
 	    fclose(bozo_logFile);
-	} else {
-	    printf("%s ", tdate);
-	    vprintf(format, ap);
 	}
     }
 }
