@@ -11,10 +11,16 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/bucoord/commands.c,v 1.14.2.1 2004/10/18 07:11:50 shadow Exp $");
+    ("$Header: /cvs/openafs/src/bucoord/commands.c,v 1.17 2005/07/11 03:22:19 shadow Exp $");
 
 #include <afs/stds.h>
+#if defined(AFS_LINUX24_ENV)
+#define _REGEX_RE_COMP
+#endif
 #include <sys/types.h>
+#if defined(AFS_LINUX24_ENV)
+#include <regex.h>
+#endif
 #include <afs/cmd.h>
 #ifdef AFS_NT40_ENV
 #include <winsock2.h>
@@ -60,11 +66,7 @@ extern statusP createStatusNode();
 char *loadFile;
 extern afs_int32 lastTaskCode;
 
-#ifdef AFS_DEC_ENV
-#define HOSTADDR(sockaddr) (sockaddr)->sin_addr.S_un.S_addr
-#else
 #define HOSTADDR(sockaddr) (sockaddr)->sin_addr.s_addr
-#endif
 
 int
 bc_EvalVolumeSet(aconfig, avs, avols, uclient)
